#!/usr/bin/env python3
from bokeh import events
from bokeh.io import curdoc
from bokeh.layouts import column, row
from bokeh.models import CustomJS, HoverTool, Panel, Tabs, Button, DateFormatter
from bokeh.models.widgets import DataTable, TableColumn
from bokeh.palettes import Colorblind, Plasma
from bokeh.plotting import ColumnDataSource, figure, output_file, show
from os import makedirs, path
from pandas import read_csv, to_datetime

import importlib.resources


def plotBSRN(filepath: str, patterns: list[str] = None, timeStart: str = None, timeEnd: str = None):

    # Import file
    yearMonth = path.basename(filepath).split("_")[0]
    bsrnFile = read_csv(filepath)

    # Truncate data between date
    timeStartIndex = None
    timeEndIndex = None
    if timeStart is not None:
        timeStartIndex = bsrnFile.timestamp[bsrnFile.timestamp == timeStart].index[0]
    if timeEnd is not None:
        timeEndIndex = bsrnFile.timestamp[bsrnFile.timestamp == timeEnd].index[0] + 1
    bsrnFile = bsrnFile[timeStartIndex:timeEndIndex]
    bsrnFile.timestamp = to_datetime(bsrnFile.timestamp, format="%Y-%m-%d %H:%M:%S")

    # Creation column data source
    sensors = bsrnFile.columns
    if patterns is not None:
        sensors = [match for match in bsrnFile.columns if any(pattern in match for pattern in patterns)]
    varNames = sensors[:]
    varNames.insert(0, "timestamp")
    bsrnData = ColumnDataSource(data=bsrnFile[varNames])
    bsrnSelect = ColumnDataSource(data={name: [] for name in varNames})
    columns = [TableColumn(field=name) for name in varNames]
    columns[0].formatter = DateFormatter(format="%Y-%m-%d %H:%M:%S")
    tableData = DataTable(
        source=bsrnSelect, columns=columns, sizing_mode="stretch_both",
        sortable=True, selectable=True, editable=False
    )

    # Selection
    with importlib.resources.path("pybsrnqc", "callbackSelect.js") as data_path:
        f = open(data_path, "r")
    js_code = f.read()
    callback = CustomJS(args=dict(bsrnData=bsrnData, bsrnSelect=bsrnSelect), code=js_code)
    bsrnData.selected.js_on_change('indices', callback)

    # Plot options
    curdoc().theme = "caliber"
    makedirs("output", exist_ok=True)
    output_file("./output/" + yearMonth + "_lines.html")
    hover = HoverTool(
        tooltips=[("Date", "$x{%F %T}"), ("Value", "$y")],
        formatters={"$x": "datetime"}
    )
    tools = [hover, "pan,wheel_zoom,box_zoom,reset,save,box_select,lasso_select"]
    colors = (Plasma[8] + Colorblind[8])[:len(sensors)]

    # Lines plot
    fig = figure(
        x_axis_label="Timestamp",
        y_axis_label="W/m^2",
        x_axis_type="datetime",
        tools=tools,
        sizing_mode="stretch_both",
        title="Visual quality control")
    for (color, sensor) in zip(colors, sensors):
        fig.line("timestamp", sensor, source=bsrnData, color=color, legend_label=sensor)
    fig.legend.click_policy = "hide"

    # Scatter plot
    fig2 = figure(
        x_axis_label="Timestamp",
        y_axis_label="W/m^2",
        x_axis_type="datetime",
        tools=tools,
        sizing_mode="stretch_both",
        title="Visual quality control")
    for (color, sensor) in zip(colors, sensors):
        fig2.scatter("timestamp", sensor, source=bsrnData, color=color, legend_label=sensor)
    fig2.legend.click_policy = "hide"

    # Download button
    button_flag = Button(label="Generate data file", button_type="danger")
    with importlib.resources.path("pybsrnqc", "callbackSave.js") as data_path:
        f = open(data_path, "r")
    js_code = f.read()
    callback = CustomJS(args=dict(bsrnSelect=bsrnSelect), code=js_code)
    button_flag.js_on_event(events.ButtonClick, callback)

    # Layout
    ltab = Panel(child=fig, title="Line")
    stab = Panel(child=fig2, title="Scatter")
    tabs = Tabs(tabs=[ltab, stab])
    layout = row(tabs, column(tableData, button_flag))

    return show(layout)



#!/usr/bin/env python3
import pandas as pd
import pvlib


def isfloat(value):
    """Checking if a string can be converted to float"""
    try:
        float(value)
        return True
    except ValueError:
        return False


def getZenith(timestamp_list, lat, lon, alt):
    """From list of timestamp, return a pandas serie that associates time with its zenith angle"""
    # get local time from timestamp row with pandas approach
    date_time_utc = pd.to_datetime(timestamp_list)
    # get Solar Zenith Angle
    pv = pvlib.solarposition.get_solarposition(date_time_utc, lat, lon, alt, pressure=None, method='nrel_numpy')
    return pv.zenith



"""
Module that open the data  as we want
"""

import os

import pandas as pd

from pybsrnqc.config import Station
from pybsrnqc.utils import getZenith


# -----------------------------------------------------------------------------------------------------------------

# Récupération de toutes les données


def open_all(path='./dataset', period=None, select_day=False, select_zenith=True):
    """ Open as a dataframe the brut data in a repository
    select_day = True means it doesn't take night hours
    select_zenith = True means it computes the SZA in the dataframe
    You have to choose the period of month you want """
    path = path + '/'
    if period is None:

        dirs = os.listdir(path)
        if '.keep' in dirs:
            dirs.remove('.keep')

    else:

        month = period[0]
        list_month = [month]

        while month != period[1]:

            if month[4:] != '12':

                month_new = str(int(month) + 1)
            else:

                year = month[:4]
                year_new = str(int(year) + 1)
                month_new = year_new + '01'

            month = month_new
            list_month.append(month)

        dirs = []

        for el in list_month:

            dirs.append(str(el) + '_brut.csv')

    print(f'Nombre de mois : {len(dirs)}')

    les_df = []

    for i in range(len(dirs)):
        df = pd.read_csv(path + dirs[i], encoding='latin-1', sep=',')
        les_df.append(df)

    fusion_df = pd.concat(les_df)

    df_fus = fusion_df.copy()

    # Zenith computation

    timestamp_list = df_fus.timestamp.to_list()
    zenith_serie = getZenith(timestamp_list, Station().LAT, Station().LON, Station().ALT)
    df_fus['SZA'] = list(zenith_serie)

    if select_zenith:
        # Zenith selection between 0° and 90°
        df_fus = df_fus.loc[(df_fus['SZA'] >= 0) & (df_fus['SZA'] <= 90)]

    if select_day:
        # Days selection only
        df_fus = df_fus.set_index('timestamp')
        df_fus.index = pd.to_datetime(df_fus.index)
        df_fus = df_fus.between_time('05:00:00', '19:05:00')

    df_fus = df_fus.dropna()

    return df_fus



import importlib.resources
import json

# Load default conf from package
with importlib.resources.path("pybsrnqc", "qcrad_conf.json") as data_path:
    with open(data_path, 'r') as f:
        loaded_json = json.load(f)


class Station:

    def __init__(self):
        self.LAT = loaded_json['STATION']['LAT']
        self.LON = loaded_json['STATION']['LON']
        self.ALT = loaded_json['STATION']['ALT']
        self.TZ = loaded_json['STATION']['TZ']


class Coef:

    def __init__(self):
        self.C1 = loaded_json['COEF']["C1"]
        self.D1 = loaded_json['COEF']["D1"]
        self.C2 = loaded_json['COEF']["C2"]
        self.D2 = loaded_json['COEF']["D1"]
        self.C3 = loaded_json['COEF']["C3"]
        self.D3 = loaded_json['COEF']["D3"]
        self.C5 = loaded_json['COEF']["C5"]
        self.D5 = loaded_json['COEF']["D5"]
        self.C6 = loaded_json['COEF']["C6"]
        self.D6 = loaded_json['COEF']["D6"]
        self.C11 = loaded_json['COEF']["C11"]
        self.D11 = loaded_json['COEF']["D11"]
        self.C12 = loaded_json['COEF']["C12"]
        self.D12 = loaded_json['COEF']["D12"]
        self.C17D = loaded_json['COEF']["C17D"]
        self.C18 = loaded_json['COEF']["C18"]
        self.C19 = loaded_json['COEF']["C19"]


class Header:

    def __init__(self):
        self.TIMESTAMP_NAME = loaded_json['HEADER']['TIMESTAMP_NAME']
        self.GSW_NAME = loaded_json['HEADER']['GSW_NAME']
        self.DIF_NAME = loaded_json['HEADER']['DIF_NAME']
        self.DIR_NAME = loaded_json['HEADER']['DIR_NAME']
        self.LWDN_NAME = loaded_json['HEADER']['LWDN_NAME']
        self.TA_NAME = loaded_json['HEADER']['TA_NAME']




from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='pybsrnqc',
    version='0.1.57',
    description='Package to study BSRN data and their quality control',
    url='https://github.com/LE2P/PyBsrnQC/tree/main/pybsrnqc',
    author='Maelle Baronnet',
    author_email='maelle.baronnet@gmail.com',
    license='MIT',
    include_package_data=True,
    packages=['pybsrnqc'],
    install_requires=['pandas',
                      'numpy>=1.17.4',
                      'matplotlib>=3.3.2',
                      'pvlib>=0.8.0',
                      'bokeh>=2.2.3',
                      'cryptography>=2.8',
                      'cassandra_driver>=3.25.0',
                      'pytz>=2019.3'
                      ],
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)
